@@ -1,5 +1,12 @@
 BEGIN;
 
+CREATE TABLE releases (
+    version         FLOAT       PRIMARY KEY,
+    installed_at    DATETIME    NOT NULL DEFAULT CURRENT_TIMESTAMP,
+    installer_name  TEXT        NOT NULL,
+    installer_email TEXT        NOT NULL
+);
+
 CREATE TABLE projects (
     project         TEXT        PRIMARY KEY,
     uri             TEXT            NULL UNIQUE,
@@ -10,6 +17,7 @@ CREATE TABLE projects (
 
 CREATE TABLE changes (
     change_id       TEXT        PRIMARY KEY,
+    script_hash     TEXT            NULL UNIQUE,
     change          TEXT        NOT NULL,
     project         TEXT        NOT NULL REFERENCES projects(project) ON UPDATE CASCADE,
     note            TEXT        NOT NULL DEFAULT '',
@@ -48,7 +56,7 @@ CREATE TABLE dependencies (
 );
 
 CREATE TABLE events (
-    event           TEXT        NOT NULL CHECK (event IN ('deploy', 'revert', 'fail')),
+    event           TEXT        NOT NULL CHECK (event IN ('deploy', 'revert', 'fail', 'merge')),
     change_id       TEXT        NOT NULL,
     change          TEXT        NOT NULL,
     project         TEXT        NOT NULL REFERENCES projects(project) ON UPDATE CASCADE,
