@@ -190,6 +190,13 @@ has planner_email => (
     default  => sub { shift->sqitch->user_email },
 );
 
+has script_hash => (
+    is       => 'ro',
+    isa      => Maybe[Str],
+    lazy     => 1,
+    builder => '_deploy_hash'
+);
+
 sub dependencies {
     my $self = shift;
     return $self->requires, $self->conflicts;
@@ -200,6 +207,15 @@ sub deploy_file {
     $self->target->deploy_dir->file( $self->path_segments );
 }
 
+sub _deploy_hash {
+    my $path = shift->deploy_file;
+    return undef unless -f $path;
+    require Digest::SHA;
+    my $sha = Digest::SHA->new(1);
+    $sha->add( $path->slurp(iomode => '<:raw') );
+    return $sha->hexdigest;
+}
+
 sub revert_file {
     my $self   = shift;
     $self->target->revert_dir->file( $self->path_segments );
@@ -427,6 +443,12 @@ Returns the path to the verify script file for the change.
 
 Returns the path to a script, for the change.
 
+=head3 C<script_hash>
+
+  my $hash = $change->script_hash;
+
+Returns the hex digest of the SHA-1 hash for the deploy script.
+
 =head3 C<rework_tags>
 
   my @tags = $change->rework_tags;
